@@ -11,6 +11,7 @@ import datetime, os
 from django.db import models
 from django.test import TestCase
 from django.core.management import call_command
+from django.core.exceptions import ImproperlyConfigured
 from django.conf import settings
 from django.conf.urls import url, patterns, include
 from django.contrib import admin
@@ -20,14 +21,14 @@ except ImportError: # django < 1.5
     from django.contrib.auth.models import User
 else:
     User = get_user_model()
-from django.utils.decorators import decorator_from_middleware
+from django.utils.decorators import decorator_from_middleware as django_decorator_from_middleware
 from django.http import HttpResponse
 from django.utils.unittest import skipUnless
 from django.utils.encoding import force_text, python_2_unicode_compatible
 
 import reversion
 from reversion.revisions import RegistrationError, RevisionManager
-from reversion.models import Revision, Version, VERSION_ADD, VERSION_CHANGE, VERSION_DELETE
+from reversion.models import Revision, Version
 from reversion.middleware import RevisionMiddleware
 
 
@@ -52,15 +53,15 @@ class ReversionTestModelBase(models.Model):
     name = models.CharField(
         max_length = 100,
     )
-    
+
     def __str__(self):
         return self.name
 
     class Meta:
         abstract = True
         app_label = "auth"  # Hack: Cannot use an app_label that is under South control, due to http://south.aeracode.org/ticket/520
-        
-        
+
+
 class ReversionTestModel1(ReversionTestModelBase):
 
     pass
@@ -72,8 +73,8 @@ def get_str_pk():
     global str_pk_gen
     str_pk_gen += 1;
     return force_text(str_pk_gen)
-    
-    
+
+
 class ReversionTestModel2(ReversionTestModelBase):
 
     id = models.CharField(
@@ -81,8 +82,8 @@ class ReversionTestModel2(ReversionTestModelBase):
         max_length = 100,
         default = get_str_pk
     )
-    
-    
+
+
 class ReversionTestModel1Proxy(ReversionTestModel1):
 
     class Meta:
@@ -94,11 +95,11 @@ class RevisionMeta(models.Model):
     revision = models.OneToOneField(Revision)
 
     age = models.IntegerField()
-    
+
     class Meta:
         app_label = "auth"  # Hack: Cannot use an app_label that is under South control, due to http://south.aeracode.org/ticket/520
-    
-    
+
+
 class RegistrationTest(TestCase):
 
     def testRegistration(self):
@@ -114,10 +115,13 @@ class RegistrationTest(TestCase):
         self.assertRaises(RegistrationError, lambda: reversion.unregister(ReversionTestModel1))
         self.assertTrue(ReversionTestModel1 not in reversion.get_registered_models())
         self.assertRaises(RegistrationError, lambda: isinstance(reversion.get_adapter(ReversionTestModel1)))
-        
+
     def testProxyRegistration(self):
         # Test error if registering proxy models.
-        self.assertRaises(RegistrationError, lambda: reversion.register(ReversionTestModel1Proxy))
+        with self.assertRaises(RegistrationError) as cm:
+            reversion.register(ReversionTestModel1Proxy)
+        self.assertEqual(str(cm.exception),
+                         "ReversionTestModel1Proxy is a proxy model, and cannot be used with django-reversion, register the parent class (ReversionTestModel1) instead.")  # noqa
 
 
 class ReversionTestBase(TestCase):
@@ -147,7 +151,7 @@ class ReversionTestBase(TestCase):
         self.user = User.objects.create(
             username = "user1",
         )
-        
+
     def tearDown(self):
         # Unregister the test models.
         reversion.unregister(ReversionTestModel1)
@@ -184,7 +188,7 @@ class InternalsTest(RevisionTestBase):
     def testRevisionsCreated(self):
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 4)
-        
+
     def testContextManager(self):
         # New revision should be created.
         with reversion.create_revision():
@@ -192,7 +196,7 @@ class InternalsTest(RevisionTestBase):
             self.test11.save()
         self.assertEqual(Revision.objects.count(), 2)
         self.assertEqual(Version.objects.count(), 5)
-    
+
     def testManualRevisionManagement(self):
         # When manage manually is on, no revisions created.
         with reversion.create_revision(manage_manually=True):
@@ -204,13 +208,13 @@ class InternalsTest(RevisionTestBase):
         reversion.default_revision_manager.save_revision([self.test11])
         self.assertEqual(Revision.objects.count(), 2)
         self.assertEqual(Version.objects.count(), 5)
-        
+
     def testEmptyRevisionNotCreated(self):
         with reversion.create_revision():
             pass
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 4)
-        
+
     def testRevisionContextAbandonedOnError(self):
         try:
             with reversion.create_revision():
@@ -221,7 +225,7 @@ class InternalsTest(RevisionTestBase):
             pass
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 4)
-        
+
     def testRevisionDecoratorAbandonedOnError(self):
         @reversion.create_revision()
         def make_revision():
@@ -234,25 +238,10 @@ class InternalsTest(RevisionTestBase):
             pass
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 4)
-        
-    def testCorrectVersionFlags(self):
-        self.assertEqual(Version.objects.filter(type=VERSION_ADD).count(), 4)
-        self.assertEqual(Version.objects.filter(type=VERSION_CHANGE).count(), 0)
-        self.assertEqual(Version.objects.filter(type=VERSION_DELETE).count(), 0)
-        with reversion.create_revision():
-            self.test11.save()
-        self.assertEqual(Version.objects.filter(type=VERSION_ADD).count(), 4)
-        self.assertEqual(Version.objects.filter(type=VERSION_CHANGE).count(), 1)
-        self.assertEqual(Version.objects.filter(type=VERSION_DELETE).count(), 0)
-        with reversion.create_revision():
-            self.test11.delete()
-        self.assertEqual(Version.objects.filter(type=VERSION_ADD).count(), 4)
-        self.assertEqual(Version.objects.filter(type=VERSION_CHANGE).count(), 1)
-        self.assertEqual(Version.objects.filter(type=VERSION_DELETE).count(), 1)
 
 
 class ApiTest(RevisionTestBase):
-    
+
     def setUp(self):
         super(ApiTest, self).setUp()
         with reversion.create_revision():
@@ -264,7 +253,7 @@ class ApiTest(RevisionTestBase):
             self.test21.save()
             self.test22.name = "model2 instance2 version2"
             self.test22.save()
-    
+
     def testRevisionSignals(self):
         pre_revision_receiver_called = []
         def pre_revision_receiver(**kwargs):
@@ -286,7 +275,7 @@ class ApiTest(RevisionTestBase):
         # Check the signals were called.
         self.assertTrue(pre_revision_receiver_called)
         self.assertTrue(post_revision_receiver_called)
-    
+
     def testCanGetForObjectReference(self):
         # Test a model with an int pk.
         versions = reversion.get_for_object_reference(ReversionTestModel1, self.test11.pk)
@@ -298,7 +287,7 @@ class ApiTest(RevisionTestBase):
         self.assertEqual(len(versions), 2)
         self.assertEqual(versions[0].field_dict["name"], "model2 instance1 version2")
         self.assertEqual(versions[1].field_dict["name"], "model2 instance1 version1")
-    
+
     def testCanGetForObject(self):
         # Test a model with an int pk.
         versions = reversion.get_for_object(self.test11)
@@ -310,7 +299,7 @@ class ApiTest(RevisionTestBase):
         self.assertEqual(len(versions), 2)
         self.assertEqual(versions[0].field_dict["name"], "model2 instance1 version2")
         self.assertEqual(versions[1].field_dict["name"], "model2 instance1 version1")
-        
+
     def testCanGetUniqueForObject(self):
         with reversion.create_revision():
             self.test11.save()
@@ -321,7 +310,7 @@ class ApiTest(RevisionTestBase):
         # Test a model with a str pk.
         self.assertEqual(reversion.get_for_object(self.test21).count(), 3)
         self.assertEqual(len(reversion.get_unique_for_object(self.test21)), 2)
-        
+
     def testCanGetForDate(self):
         with self.settings(USE_TZ=True):
             now = datetime.datetime.now(UTC())
@@ -333,7 +322,7 @@ class ApiTest(RevisionTestBase):
             version = reversion.get_for_date(self.test21, now)
             self.assertEqual(version.field_dict["name"], "model2 instance1 version2")
             self.assertRaises(Version.DoesNotExist, lambda: reversion.get_for_date(self.test21, datetime.datetime(1970, 1, 1, tzinfo=UTC())))
-        
+
     def testCanGetDeleted(self):
         with reversion.create_revision():
             self.test11.delete()
@@ -346,18 +335,18 @@ class ApiTest(RevisionTestBase):
         versions = reversion.get_deleted(ReversionTestModel2)
         self.assertEqual(len(versions), 1)
         self.assertEqual(versions[0].field_dict["name"], "model2 instance1 version2")
-        
+
     def testCanRevertVersion(self):
         reversion.get_for_object(self.test11)[1].revert()
         self.assertEqual(ReversionTestModel1.objects.get(id=self.test11.pk).name, "model1 instance1 version1")
-        
+
     def testCanRevertRevision(self):
         reversion.get_for_object(self.test11)[1].revision.revert()
         self.assertEqual(ReversionTestModel1.objects.get(id=self.test11.pk).name, "model1 instance1 version1")
         self.assertEqual(ReversionTestModel1.objects.get(id=self.test12.pk).name, "model1 instance2 version1")
         self.assertEqual(ReversionTestModel2.objects.get(id=self.test22.pk).name, "model2 instance2 version1")
         self.assertEqual(ReversionTestModel2.objects.get(id=self.test22.pk).name, "model2 instance2 version1")
-    
+
     def testCanRevertRevisionWithDeletedVersions(self):
         self.assertEqual(ReversionTestModel1.objects.count(), 2)
         self.assertEqual(ReversionTestModel2.objects.count(), 2)
@@ -387,7 +376,7 @@ class ApiTest(RevisionTestBase):
         reversion.get_for_object(self.test11)[2].revision.revert()
         self.assertEqual(ReversionTestModel1.objects.count(), 2)
         self.assertEqual(ReversionTestModel2.objects.count(), 2)
-    
+
     def testCanSaveIgnoringDuplicates(self):
         with reversion.create_revision():
             self.test11.save()
@@ -404,7 +393,7 @@ class ApiTest(RevisionTestBase):
             self.assertFalse(reversion.get_ignore_duplicates())
             reversion.set_ignore_duplicates(True)
         self.assertEqual(reversion.get_for_object(self.test11).count(), 3)
-        
+
     def testCanAddMetaToRevision(self):
         # Create a revision with lots of meta data.
         with reversion.create_revision():
@@ -424,8 +413,8 @@ class ApiTest(RevisionTestBase):
 class ReversionTestModel1Child(ReversionTestModel1):
 
     pass
-    
-    
+
+
 class MultiTableInheritanceApiTest(RevisionTestBase):
 
     def setUp(self):
@@ -435,10 +424,10 @@ class MultiTableInheritanceApiTest(RevisionTestBase):
             self.testchild1 = ReversionTestModel1Child.objects.create(
                 name = "modelchild1 instance1 version 1",
             )
-    
+
     def testCanRetreiveFullFieldDict(self):
         self.assertEqual(reversion.get_for_object(self.testchild1)[0].field_dict["name"], "modelchild1 instance1 version 1")
-    
+
     def tearDown(self):
         super(MultiTableInheritanceApiTest, self).tearDown()
         del self.testchild1
@@ -449,12 +438,12 @@ class TestFollowModel(ReversionTestModelBase):
     test_model_1 = models.ForeignKey(
         ReversionTestModel1,
     )
-    
+
     test_model_2s = models.ManyToManyField(
         ReversionTestModel2,
     )
-    
-    
+
+
 class FollowModelsTest(ReversionTestBase):
 
     @reversion.create_revision()
@@ -468,7 +457,7 @@ class FollowModelsTest(ReversionTestBase):
             test_model_1 = self.test11,
         )
         self.follow1.test_model_2s.add(self.test21, self.test22)
-    
+
     def testRelationsFollowed(self):
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 5)
@@ -476,7 +465,7 @@ class FollowModelsTest(ReversionTestBase):
             self.follow1.save()
         self.assertEqual(Revision.objects.count(), 2)
         self.assertEqual(Version.objects.count(), 9)
-    
+
     def testRevertWithDelete(self):
         with reversion.create_revision():
             test23 = ReversionTestModel2.objects.create(
@@ -502,7 +491,7 @@ class FollowModelsTest(ReversionTestBase):
         # Roll back to a revision where a delete flag is present.
         reversion.get_for_object(self.follow1)[0].revision.revert(delete=True)
         self.assertEqual(self.follow1.test_model_2s.all().count(), 2)
-    
+
     def testReverseRelationsFollowed(self):
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 5)
@@ -510,7 +499,7 @@ class FollowModelsTest(ReversionTestBase):
             self.test11.save()
         self.assertEqual(Revision.objects.count(), 2)
         self.assertEqual(Version.objects.count(), 9)
-    
+
     def testReverseFollowRevertWithDelete(self):
         with reversion.create_revision():
             follow2 = TestFollowModel.objects.create(
@@ -522,7 +511,7 @@ class FollowModelsTest(ReversionTestBase):
         reversion.get_for_object(self.test11)[1].revision.revert(delete=True)
         self.assertEqual(TestFollowModel.objects.count(), 1)
         self.assertRaises(TestFollowModel.DoesNotExist, lambda: TestFollowModel.objects.get(id=follow2_pk))
-    
+
     def testRecoverDeleted(self):
         # Delete the test model.
         with reversion.create_revision():
@@ -535,7 +524,7 @@ class FollowModelsTest(ReversionTestBase):
         # Make sure it was recovered.
         self.assertEqual(TestFollowModel.objects.count(), 1)
         self.assertEqual(ReversionTestModel1.objects.count(), 2)
-    
+
     def tearDown(self):
         reversion.unregister(TestFollowModel)
         TestFollowModel.objects.all().delete()
@@ -547,27 +536,27 @@ excluded_revision_manager = RevisionManager("excluded")
 
 
 class ExcludedFieldsTest(RevisionTestBase):
-    
+
     def setUp(self):
         excluded_revision_manager.register(ReversionTestModel1, fields=("id",))
         excluded_revision_manager.register(ReversionTestModel2, exclude=("name",))
         super(ExcludedFieldsTest, self).setUp()
-    
+
     def testExcludedRevisionManagerIsSeparate(self):
         self.assertEqual(excluded_revision_manager.get_for_object(self.test11).count(), 1)
-        
+
     def testExcludedFieldsAreRespected(self):
         self.assertEqual(excluded_revision_manager.get_for_object(self.test11)[0].field_dict["id"], self.test11.id)
         self.assertEqual(excluded_revision_manager.get_for_object(self.test11)[0].field_dict["name"], "")
         self.assertEqual(excluded_revision_manager.get_for_object(self.test21)[0].field_dict["id"], self.test21.id)
         self.assertEqual(excluded_revision_manager.get_for_object(self.test21)[0].field_dict["name"], "")
-        
+
     def tearDown(self):
         super(ExcludedFieldsTest, self).tearDown()
         excluded_revision_manager.unregister(ReversionTestModel1)
         excluded_revision_manager.unregister(ReversionTestModel2)
-        
-        
+
+
 class CreateInitialRevisionsTest(ReversionTestBase):
 
     def testCreateInitialRevisions(self):
@@ -581,12 +570,12 @@ class CreateInitialRevisionsTest(ReversionTestBase):
         call_command("createinitialrevisions")
         self.assertEqual(Revision.objects.count(), revcount)
         self.assertEqual(Version.objects.count(), vercount)
-        
+
     def testCreateInitialRevisionsSpecificApps(self):
         call_command("createinitialrevisions", "auth")
         self.assertEqual(Revision.objects.count(), 4)
         self.assertEqual(Version.objects.count(), 4)
-        
+
     def testCreateInitialRevisionsSpecificModels(self):
         call_command("createinitialrevisions", "auth.ReversionTestModel1")
         self.assertEqual(Revision.objects.count(), 2)
@@ -594,13 +583,50 @@ class CreateInitialRevisionsTest(ReversionTestBase):
         call_command("createinitialrevisions", "auth.ReversionTestModel2")
         self.assertEqual(Revision.objects.count(), 4)
         self.assertEqual(Version.objects.count(), 4)
-        
+
     def testCreateInitialRevisionsSpecificComment(self):
         call_command("createinitialrevisions", comment="Foo bar")
         self.assertEqual(Revision.objects.all()[0].comment, "Foo bar")
 
 
-# Tests for reversion functionality that's tied to requests.        
+# Tests for reversion functionality that's tied to requests.
+
+# RevisionMiddleware is tested by applying it as a decorator to various view
+# functions. Django's decorator_from_middleware() utility function does the
+# trick of converting a middleware class to a decorator. However, in projects
+# that include the middleware in the MIDDLEWARE_CLASSES setting, the wrapped
+# view function is processed twice by the middleware. When RevisionMiddleware
+# processes a function twice, an ImproperlyConfigured exception is raised.
+# Thus, using Django's definition of decorator_from_middleware() can prevent
+# reversion integration tests from passing in projects that include
+# RevisionMiddleware in MIDDLEWARE_CLASSES.
+#
+# To avoid this problem, we redefine decorator_from_middleware() to return a
+# decorator that does not reapply the middleware if it is in
+# MIDDLEWARE_CLASSES.  @decorator_from_middleware(RevisionMiddleware) is then
+# used to wrap almost all RevisionMiddleware test views. The only exception is
+# double_middleware_revision_view(), which needs to be doubly processed by
+# RevisionMiddleware.  This view is wrapped twice with
+# @django_decorator_from_middleware(RevisionMiddleware), where
+# django_decorator_from_middleware() is imported as Django's definition of
+# decorator_from_middleware().
+
+revision_middleware_django_decorator = django_decorator_from_middleware(RevisionMiddleware)
+
+def decorator_from_middleware(middleware_class):
+    """
+    This is a wrapper around django.utils.decorators.decorator_from_middleware
+    (imported as django_decorator_from_middleware). If the middleware class is
+    not loaded via MIDDLEWARE_CLASSES in the project settings, then the
+    middleware decorator is returned. However, if the middleware is already
+    loaded, then an identity decorator is returned instead, so that the
+    middleware does not process the view function twice.
+    """
+    middleware_path = "%s.%s" % (middleware_class.__module__,
+                                 middleware_class.__name__)
+    if middleware_path in settings.MIDDLEWARE_CLASSES:
+        return lambda view_func: view_func
+    return django_decorator_from_middleware(middleware_class)
 
 revision_middleware_decorator = decorator_from_middleware(RevisionMiddleware)
 
@@ -620,8 +646,8 @@ def save_revision_view(request):
         name = "model2 instance4 version1",
     )
     return HttpResponse("OK")
-    
-    
+
+
 # A dumb view that borks a revision.
 @revision_middleware_decorator
 def error_revision_view(request):
@@ -640,6 +666,13 @@ def error_revision_view(request):
     raise Exception("Foo")
 
 
+# A dumb view that has two revision middlewares.
+@revision_middleware_django_decorator
+@revision_middleware_django_decorator
+def double_middleware_revision_view(request):
+    raise Exception("Foo")
+
+
 site = admin.AdminSite()
 
 
@@ -648,7 +681,7 @@ class ParentTestAdminModel(models.Model):
     parent_name = models.CharField(
         max_length = 200,
     )
-    
+
     class Meta:
         app_label = "auth"  # Hack: Cannot use an app_label that is under South control, due to http://south.aeracode.org/ticket/520
 
@@ -659,10 +692,10 @@ class ChildTestAdminModel(ParentTestAdminModel):
     child_name = models.CharField(
         max_length = 200,
     )
-    
+
     def __str__(self):
         return self.child_name
-    
+
     class Meta:
         app_label = "auth"  # Hack: Cannot use an app_label that is under South control, due to http://south.aeracode.org/ticket/520
 
@@ -709,12 +742,30 @@ class InlineTestParentModelAdmin(reversion.VersionAdmin):
 site.register(InlineTestParentModel, InlineTestParentModelAdmin)
 
 
+# Test that reversion handles unrelated inlines.
+# Issue https://github.com/etianen/django-reversion/issues/277
+class InlineTestUnrelatedParentModel(models.Model):
+    pass
+
+class InlineTestUnrelatedChildModel(models.Model):
+    pass
+
+class InlineTestUnrelatedChildModelInline(admin.TabularInline):
+    model = InlineTestUnrelatedChildModel
+
+class InlineTestUnrelatedParentModelAdmin(reversion.VersionAdmin):
+    inlines = (InlineTestUnrelatedChildModelInline, )
+site.register(InlineTestUnrelatedParentModel, InlineTestUnrelatedParentModelAdmin)
+
+
 urlpatterns = patterns("",
 
     url("^success/$", save_revision_view),
-    
+
     url("^error/$", error_revision_view),
-    
+
+    url("^double/$", double_middleware_revision_view),
+
     url("^admin/", include(site.get_urls(), namespace="admin")),
 
 )
@@ -730,7 +781,7 @@ class RevisionMiddlewareTest(ReversionTestBase):
         self.client.get("/success/")
         self.assertEqual(Revision.objects.count(), 1)
         self.assertEqual(Version.objects.count(), 4)
-        
+
     def testRevisionMiddlewareInvalidatesRevisionOnError(self):
         self.assertEqual(Revision.objects.count(), 0)
         self.assertEqual(Version.objects.count(), 0)
@@ -738,6 +789,9 @@ class RevisionMiddlewareTest(ReversionTestBase):
         self.assertEqual(Revision.objects.count(), 0)
         self.assertEqual(Version.objects.count(), 0)
 
+    def testRevisionMiddlewareErrorOnDoubleMiddleware(self):
+        self.assertRaises(ImproperlyConfigured, lambda: self.client.get("/double/"))
+
 
 class VersionAdminTest(TestCase):
 
@@ -912,17 +966,17 @@ except ImportError:
     can_test_patch = False
 else:
     can_test_patch = True
-    
-    
+
+
 class PatchTest(RevisionTestBase):
-    
+
     def setUp(self):
         super(PatchTest, self).setUp()
         with reversion.create_revision():
             self.test11.name = "model1 instance1 version2"
             self.test11.save()
-        self.version2, self.version1 = reversion.get_for_object(self.test11) 
-    
+        self.version2, self.version1 = reversion.get_for_object(self.test11)
+
     @skipUnless(can_test_patch, "Diff match patch library not installed")
     def testCanGeneratePatch(self):
         self.assertEqual(
@@ -930,14 +984,29 @@ class PatchTest(RevisionTestBase):
             "@@ -17,9 +17,9 @@\n  version\n-1\n+2\n",
         )
 
-    @skipUnless(can_test_patch, "Diff match patch library not installed")    
+    @skipUnless(can_test_patch, "Diff match patch library not installed")
     def testCanGeneratePathHtml(self):
         self.assertEqual(
             generate_patch_html(self.version1, self.version2, "name"),
-            u'<span>model1 instance1 version</span><del style="background:#ffe6e6;">1</del><ins style="background:#e6ffe6;">2</ins>',
+            '<span>model1 instance1 version</span><del style="background:#ffe6e6;">1</del><ins style="background:#e6ffe6;">2</ins>',
         )
-                         
+
     def tearDown(self):
         super(PatchTest, self).tearDown()
         del self.version1
-        del self.version2
\ No newline at end of file
+        del self.version2
+
+
+# test preserve deleted User Revisions
+class DeleteUserTest(RevisionTestBase):
+
+    def testDeleteUser(self):
+        self.assertEqual(Revision.objects.count(), 1)
+        self.assertEqual(Version.objects.count(), 4)
+        rev = Revision.objects.all()[0]
+        rev.user = self.user
+        rev.save()
+        self.user.delete()
+        self.assertEqual(Revision.objects.count(), 1)
+        self.assertEqual(Version.objects.count(), 4)
+
